#include <ctype.h>
#include "mep.h"

typedef struct field {
  char *string;
  struct field *next;
} field_t;

typedef struct fields {
  field_t *first;
  field_t *last;
} fields_t;

typedef struct header {
  char *hd_subject;
  char *hd_sender;
  char *hd_recipient;
  char *hd_from;
  fields_t hd_to;
  fields_t hd_copy;
  char *hd_verify;
  char *hd_deliver;
  char *hd_reply;
  fields_t hd_track;
} header_t;

header_t head = {
  NULL, NULL, NULL, NULL, {NULL, NULL},
  {NULL, NULL}, NULL, NULL, NULL, {NULL, NULL}
};

char *getverify(void)
{
  return head.hd_verify;
}

char *line(char *name, char *string)
{
  char *temp;
  int len;

  if (strncmp(name, string, len = strlen(name)) != 0)
    return NULL;

  for (string += len; *string == ' '; string++);

  if ((temp = strstr(string, "\r\n")) != NULL)
    *temp = '\0';

  return strdup(string);
}

void addstring(char *name, char *string, char **fields)
{
  if (*fields != NULL)
    free(*fields);

  *fields = line(name, string);
}

void addfield(char *name, char *string, fields_t *fields)
{
  field_t *temp;

  if ((string = line(name, string)) != NULL) {
    temp = (field_t *) malloc(sizeof(field_t));
    temp->string = string;
    temp->next = NULL;

    if (fields->last == NULL)
      fields->first = fields->last = temp;
    else
      fields->last = fields->last->next = temp;
  }
}

void scan(char *string)
{
  switch (tolower(string[0])) {
  case 'c':
    addfield("Copy: ", string, &head.hd_copy);
    break;
  case 'd':
    addstring("Deliver: ", string, &head.hd_deliver);
    break;
  case 'f':
    addstring("From: ", string, &head.hd_from);
    break;
  case 'r':
    switch (tolower(string[2])) {
    case 'c':
      addstring("Recipient: ", string, &head.hd_recipient);
      break;
    case 'p':
      addstring("Reply: ", string, &head.hd_reply);
      break;
    }
    break;
  case 's':
    switch (tolower(string[1])) {
    case 'e':
      addstring("Sender: ", string, &head.hd_sender);
      break;
    case 'u':
      addstring("Subject: ", string, &head.hd_subject);
      break;
    }
    break;
  case 't':
    switch (tolower(string[1])) {
    case 'o':
      addfield("To: ", string, &head.hd_to);
      break;
    case 'r':
      addfield("Track: ", string, &head.hd_track);
    }
    break;
  case 'v':
    addstring("Verify: ", string, &head.hd_verify);
    break;
  }
}

void scanheader(int desc)
{
  char *field = NULL, *string = NULL, *crlf;
  int len = 0;

  while (len != 2) {
    while (string == NULL ||
	   (crlf = strstr(string, "\r\n")) == NULL)
      string = getstring(desc, 1);

    len = crlf - string + 2;
    field = (char *) realloc(field, (len + 1) * sizeof(char));
    strncpy(field, string, len);
    field[len] = '\0';
    string += len;

    scan(field);
  }

  free(field);
}

void putstring(int file, char *name, char *string)
{
  if (string != NULL) {
    write(file, name, strlen(name));
    write(file, string, strlen(string));
    write(file, "\n", 1);
  }
}

void putfields(int file, char *name, fields_t fields)
{
  field_t *temp;

  for (temp = fields.first; temp != NULL; temp = temp->next) {
    write(file, name, strlen(name));
    write(file, temp->string, strlen(temp->string));
    write(file, "\n", 1);
  }
}

void writeheader(int file)
{
  putstring(file, "Subject: ", head.hd_subject);
  putstring(file, "Sender: ", head.hd_sender);
  putstring(file, "Recipient: ", head.hd_recipient);
  putstring(file, "From: ", head.hd_from);
  putfields(file, "To: ", head.hd_to);
  putfields(file, "Copy: ", head.hd_copy);
  putstring(file, "Verify: ", head.hd_verify);
  putstring(file, "Deliver: ", head.hd_deliver);
  putstring(file, "Reply: ", head.hd_reply);
  putfields(file, "Track: ", head.hd_track);
}

void clearstring(char **string)
{
  free(*string);

  *string = NULL;
}

void clearfields(fields_t *fields)
{
  field_t *next, *temp;

  for (temp = fields->first; temp != NULL; temp = next) {
    next = temp->next;
    free(temp->string);
    free(temp);
  }

  fields->first = fields->last = NULL;
}

void clearheader(void)
{
  clearstring(&head.hd_subject);
  clearstring(&head.hd_sender);
  clearstring(&head.hd_recipient);
  clearstring(&head.hd_from);
  clearfields(&head.hd_to);
  clearfields(&head.hd_copy);
  clearstring(&head.hd_verify);
  clearstring(&head.hd_deliver);
  clearstring(&head.hd_reply);
  clearfields(&head.hd_track);
}
