#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include <pthread.h>
#include "mep.h"

#define FLAGS O_CREAT|O_EXCL|O_WRONLY|O_TRUNC
#define MODE S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH	/* 0644 */
#define DMODE MODE|S_IXUSR|S_IXGRP|S_IXOTH	/* 0755 */

#define pchar(x) (i = x % 36) > 9 ? 0x57 + i : 0x30 + i

extern pthread_key_t key;
char *id;

char *dirname(int time)
{
  static int count = 0, oldtime = 0;
  int i, pos = 9;
  char string[pos];

  if (oldtime != time) {
    count = 0;
    oldtime = time;
  }

  string[--pos] = '\0';
  string[--pos] = pchar(count);
  string[--pos] = pchar(count++ / 36);

  while (pos > 0) {
    string[--pos] = pchar(time);
    time /= 36;
  }

  return strdup(string);
}

void verify(tsd_t *tsd)
{
  unsigned char digest[16];
  char md5[32], *buffer;
  int i, sock;

  MD5Final(digest, &tsd->context);
  for (i = 0; i < 16; i++)
    sprintf(md5 + 2 * i, "%02x", digest[i]);

  buffer = (char *) malloc((50 + strlen(id)) * sizeof(char));

  if ((sock = getmxsocket(getverify())) != 0) {
    sprintf(buffer, "INIT MEP/0.9\r\n");
    send(sock, buffer, strlen(buffer), 0);

    free(getstring(sock, 1));

    sprintf(buffer, "VERI FINA %s MD5 %s\r\n", id, md5);
    send(sock, buffer, strlen(buffer), 0);

    free(getstring(sock, 1));

    sprintf(buffer, "QUIT\r\n");
    send(sock, buffer, strlen(buffer), 0);

    free(getstring(sock, 1));

    close(sock);
  }

  free(buffer);
  free(id);
  tsd->send = 0;
  clearheader();
}

void mep_quit(char *string)
{
  tsd_t *tsd;

  tsd = (tsd_t *) pthread_getspecific(key);

  if (tsd->send == 1)
    verify(tsd);

  sendout("OK\r\n");

  close(tsd->desc);
}

void mep_send(char *string)
{
  static int num;
  char path[PATH_MAX + 1], filename[PATH_MAX + 1];
  char buffer[CHUNK_SIZE], *temp;
  int file, declared, bytes, btotal;
  time_t t;
  tsd_t *tsd;

  tsd = (tsd_t *) pthread_getspecific(key);

  if (strncasecmp(string, "HEAD", 4) == 0) {
    if (tsd->send == 1)
      verify(tsd);

    string += 4;
    while (*(++string) == ' ');

    if ((temp = strstr(string, "\r\n")) != NULL)
      *temp = '\0';

    id = strdup(string);

    num = 1;
    tsd->send = 1;
    MD5Init(&tsd->context);

    time(&t);
    sprintf(path, "%s/%s", SPOOL_PATH, temp = dirname(t));
    mkdir(path, DMODE);
    free(temp);

    sendout("ACCE\r\n");

    scanheader(tsd->desc);
    snprintf(filename, PATH_MAX, "%s/0_", path);
    if ((file = open(filename, FLAGS, MODE)) < 0)
      return;
    writeheader(file);
    close(file);

    sendout("OK\r\n");

    return;
  }

  if (tsd->send == 1 && strncasecmp(string, "PART", 4) == 0 &&
      (declared = atoi(string = strip(string, ' '))) != 0) {
    btotal = 0;

    if ((temp = strstr(string, "\r\n")) != NULL)
      *temp = '\0';

    string = strip(strip(string, ' '), ' ');
    snprintf(filename, PATH_MAX, "%s/%d_%s", path, num++, string);

    sendout("ACCE\r\n");

    if ((file = open(filename, FLAGS, MODE)) < 0)
      return;

    while ((bytes = recv(tsd->desc, buffer, CHUNK_SIZE, 0)) != 0) {
      write(file, buffer, bytes);
      MD5Update(&tsd->context, buffer, bytes);

      if ((btotal += bytes) >= declared)
	break;
    }

    close(file);

    if (btotal == declared) {
      sprintf(buffer, "OK %d bytes\r\n", btotal);
      sendout(buffer);
    } else {
      sprintf(buffer, "ERRO read %d bytes\r\n", btotal);
      sendout(buffer);
    }

    return;
  }

  sendout("ERRO\r\n");
}
